@@ -14,9 +14,9 @@ use itertools::Itertools;
 use crate::{all_imports::RInts, pyo3_prelude::*, sv, util::BitMask128};
 
 use super::{
-    computational_node::EinsumArgs, deep_map_unwrap, evaluate_fn, prelude::*,
-    py_circuit_items::PY_CIRCUIT_ITEMS, Add, Concat, Einsum, GeneralFunction, HashBytes, Index,
-    Rearrange, ScalarConstant, Scatter, Symbol,
+    circuit_optimizer::OptimizationContext, computational_node::EinsumArgs, deep_map_op_context,
+    evaluate_fn, prelude::*, py_circuit_items::PY_CIRCUIT_ITEMS, Add, Concat, Einsum,
+    GeneralFunction, HashBytes, Index, Rearrange, ScalarConstant, Scatter, Symbol,
 };
 use crate::{
     filter_by_variant,
@@ -29,7 +29,7 @@ use crate::{
     unwrap,
     util::{
         filter_out_idx, filter_to_idx, intersection_all, inverse_permutation, is_unique,
-        unique_to_appearance, AsOp, EinInt,
+        unique_to_appearance, EinInt,
     },
 };
 
@@ -309,24 +309,46 @@ pub fn einsum_nest_path(einsum: &Einsum, path: Vec<Vec<usize>>) -> Einsum {
 }
 
 #[pyfunction]
-pub fn einsum_nest_optimize(einsum: &Einsum) -> Einsum {
+#[pyo3(name = "einsum_nest_optimize")]
+pub fn einsum_nest_optimize_py(einsum: &Einsum) -> Option<Einsum> {
+    einsum_nest_optimize(
+        einsum,
+        &mut OptimizationContext::new_settings_circuit(Default::default(), einsum.clone().rc()),
+    )
+}
+
+pub fn einsum_nest_optimize(einsum: &Einsum, context: &mut OptimizationContext) -> Option<Einsum> {
     // by default dont pre-contract subsets of arrayconstants
     let path = optimize_einsum_spec_cached(
         einsum.get_spec(),
         None,
-        None,
+        Some(context.cache.max_single_tensor_numel),
         None,
         Some(filter_to_idx(&einsum.args, &|(x, _)| {
             matches!(&***x, Circuit::ArrayConstant(_))
         })),
-    );
+    )?;
     // let path = einsum.get_spec().optimize_dp(None,None,None);
-    einsum_nest_path(einsum, path)
+    Some(einsum_nest_path(einsum, path))
 }
 
 #[pyfunction]
-pub fn deep_optimize_einsums(circ: CircuitRc) -> CircuitRc {
-    deep_map_unwrap(&circ, |x| x.map_or_clone(einsum_nest_optimize))
+#[pyo3(name = "deep_optimize_einsums")]
+pub fn deep_optimize_einsums_py(circ: CircuitRc) -> CircuitRc {
+    deep_optimize_einsums(circ, &mut Default::default())
+}
+
+pub fn deep_optimize_einsums(circ: CircuitRc, context: &mut OptimizationContext) -> CircuitRc {
+    deep_map_op_context(
+        &circ,
+        &mut |x, c| match x {
+            Circuit::Einsum(einsum) => einsum_nest_optimize(&einsum, c).map(|z| z.rc()),
+            _ => None,
+        },
+        context,
+        &mut Default::default(),
+    )
+    .unwrap_or(circ)
 }
 
 /// corresponds to python function `construct_diag`
@@ -731,11 +753,11 @@ pub fn einsum_pull_removable_axes(einsum: &Einsum) -> Option<CircuitRc> {
 
 #[pyfunction]
 pub fn add_pull_removable_axes(add: &Add, remove_non_common_axes: bool) -> Option<CircuitRc> {
+    let mut removed_any_non_one_axes = false;
     let removable_axes: Vec<HashSet<usize>> = add
-        .nodes
+        .nodes_and_rank_differences()
         .iter()
-        .map(|x| {
-            let rank_difference = add.info().rank() - x.info().rank();
+        .map(|(x, rank_difference)| {
             let pre_broadcast = get_removable_axes(x);
             let pre_broadcast_with_ones: HashSet<usize> = pre_broadcast
                 .union(
@@ -744,27 +766,36 @@ pub fn add_pull_removable_axes(add: &Add, remove_non_common_axes: bool) -> Optio
                         .iter()
                         .enumerate()
                         .filter_map(|(i, l)| {
-                            if *l == add.info().shape[i + rank_difference] {
-                                None
-                            } else {
+                            if *l != add.info().shape[i + rank_difference] {
                                 Some(i)
+                            } else {
+                                None
                             }
                         })
                         .collect(),
                 )
                 .copied()
                 .collect();
-            // todo add 1 axes to removable
+            if remove_non_common_axes
+                && !removed_any_non_one_axes
+                && pre_broadcast_with_ones
+                    .iter()
+                    .any(|i| x.info().shape[*i] != 1)
+            {
+                removed_any_non_one_axes = true;
+            }
             pre_broadcast_with_ones
                 .iter()
                 .map(|i| i + rank_difference)
-                .chain(0..rank_difference)
+                .chain(0..*rank_difference)
                 .collect()
         })
         .collect();
 
     let intersection = intersection_all(&removable_axes);
-    if intersection.is_empty() {
+    if (intersection.is_empty() && !remove_non_common_axes)
+        || (remove_non_common_axes && !removed_any_non_one_axes)
+    {
         return None;
     }
     let post_rearrange_spec = RearrangeSpec::new_canon(
@@ -775,21 +806,23 @@ pub fn add_pull_removable_axes(add: &Add, remove_non_common_axes: bool) -> Optio
         (0..add.info().rank()).map(|x| sv![x as u8]).collect(),
         shape_to_op_shape(&add.info().shape),
     );
-    let new_operands = zip(&add.nodes, removable_axes)
-        .map(|(node, removable_here_base)| {
+    let new_operands = zip(add.nodes_and_rank_differences(), removable_axes)
+        .map(|((node, _rank_dif), removable_here_base)| {
             let rank_difference = add.info().rank() - node.info().rank();
             if !remove_non_common_axes {
                 let removable_common_in_rank: HashSet<usize> = intersection
                     .iter()
                     .filter_map(|i| i.checked_sub(rank_difference))
                     .collect();
-                remove_axes(node, &removable_common_in_rank).unwrap()
+                remove_axes(&node, &removable_common_in_rank).unwrap()
             } else {
                 let removable_here = removable_here_base
                     .iter()
                     .filter_map(|i| i.checked_sub(rank_difference))
                     .collect();
-                let raw_removed = remove_axes(node, &removable_here).unwrap();
+                let raw_removed = remove_axes(&node, &removable_here).unwrap();
+
+                // let removable_here_to_add_back = removable_here.iter().filter(|i|!intersection.contains(i+rank_dif))
 
                 let mut output_ints: RInts = sv![];
                 let mut count = 0;
@@ -851,6 +884,8 @@ pub fn distribute(einsum: &Einsum, operand_idx: usize, do_broadcasts: bool) -> O
             add = add_make_broadcasts_explicit(&add).unwrap();
         }
         if add.has_broadcast() || add.nodes.is_empty() {
+            println!("distribute failed");
+            add.compiler_print();
             return None;
         }
         let summands = add
