@@ -1,6 +1,11 @@
+use std::collections::HashMap;
+
 use crate::pyo3_prelude::*;
 
-use super::{deep_map_unwrap, prelude::*, Add, Einsum, Index, Rearrange};
+use super::{
+    circuit_optimizer::OptimizationContext, deep_map_op_context,
+    prelude::*, Add, Einsum, Index, Rearrange,
+};
 
 /// takes circuitrc bc convenient
 pub fn numel_sort_key(node: &CircuitRc) -> Vec<u8> {
@@ -16,12 +21,12 @@ pub fn numel_sort_key(node: &CircuitRc) -> Vec<u8> {
 #[pyfunction]
 #[pyo3(name = "canonicalize_node")]
 pub fn canonicalize_node_py(circuit: CircuitRc) -> CircuitRc {
-    canonicalize_node(&circuit)
+    canonicalize_node_op(&circuit).unwrap_or(circuit.clone())
 }
 
-pub fn canonicalize_node(circuit: &Circuit) -> CircuitRc {
+pub fn canonicalize_node_op(circuit: &Circuit) -> Option<CircuitRc> {
     match &*circuit {
-        Circuit::Rearrange(rearrange) => Rearrange::try_new(
+        Circuit::Rearrange(rearrange) => Some(Rearrange::nrc(
             rearrange.node.clone(),
             rearrange
                 .spec
@@ -29,34 +34,43 @@ pub fn canonicalize_node(circuit: &Circuit) -> CircuitRc {
                 .unwrap()
                 .canonicalize(true),
             circuit.name_cloned(),
-        )
-        .unwrap()
-        .rc(),
-        Circuit::Index(index) => Index::try_new(
+        )),
+        Circuit::Index(index) => Some(Index::nrc(
             index.node.clone(),
             index.index.canonicalize(&index.node.info().shape),
             index.name_cloned(),
-        )
-        .unwrap()
-        .rc(),
+        )),
         Circuit::Add(add) => {
             let mut nodes_sorted = add.nodes.clone();
             nodes_sorted.sort_by_key(numel_sort_key);
-            Add::try_new(nodes_sorted, add.name_cloned()).unwrap().rc()
+            Some(Add::nrc(nodes_sorted, add.name_cloned()))
         }
         Circuit::Einsum(einsum) => {
             let mut args_sorted = einsum.args.clone();
             args_sorted.sort_by_key(|(node, _ints)| numel_sort_key(node));
-            Einsum::try_new(args_sorted, einsum.out_axes.clone(), einsum.name_cloned())
-                .unwrap()
-                .normalize_ints()
-                .rc()
+            Some(
+                Einsum::try_new(args_sorted, einsum.out_axes.clone(), einsum.name_cloned())
+                    .unwrap()
+                    .normalize_ints()
+                    .rc(),
+            )
         }
-        _ => circuit.clone().rc(),
+        _ => None,
     }
 }
 
 #[pyfunction]
-pub fn deep_canonicalize(circuit: CircuitRc) -> CircuitRc {
-    deep_map_unwrap(&circuit, &canonicalize_node)
+#[pyo3(name = "deep_canonicalize")]
+pub fn deep_canonicalize_py(circuit: CircuitRc) -> CircuitRc {
+    deep_canonicalize(circuit, &mut Default::default())
+}
+
+pub fn deep_canonicalize(circuit: CircuitRc, context: &mut OptimizationContext) -> CircuitRc {
+    deep_map_op_context(
+        &circuit,
+        &|x, _c: &mut HashMap<(), ()>| canonicalize_node_op(x),
+        &mut HashMap::<(), ()>::new(),
+        &mut context.cache.canonicalized,
+    )
+    .unwrap_or(circuit.clone())
 }
