@@ -12,7 +12,7 @@ use super::{
     debugging::{create_circuit_test, randn_ablate_great_grandchildren},
     deep_rewrite::deep_heuristic_nest_adds,
     deep_rewrite::{compiler_simp, deep_push_down_index},
-    flat_concat,
+    flat_concat, get_compatible_dtype,
     scheduled_execution::{
         circuit_to_schedule, circuit_to_schedule_naive_toposort, Schedule, SchedulingError,
     },
@@ -33,9 +33,11 @@ pub struct OptimizationSettings {
     #[pyo3(get, set)]
     pub max_memory: usize,
     #[pyo3(get, set)]
+    pub max_single_tensor_memory: usize,
+    #[pyo3(get, set)]
     pub scheduling_num_mem_chunks: usize,
     #[pyo3(get, set)]
-    pub distribute_min_size: usize,
+    pub distribute_min_size: Option<usize>,
     #[pyo3(get, set)]
     pub scheduling_naive: bool,
     #[pyo3(get, set)]
@@ -66,8 +68,9 @@ impl Default for OptimizationSettings {
         Self {
             verbose: 0,
             max_memory: 9_000_000_000,
+            max_single_tensor_memory: 9_000_000_000,
             scheduling_num_mem_chunks: 200,
-            distribute_min_size: 800_000_000,
+            distribute_min_size: None,
             scheduling_naive: false,
             scheduling_simplify: true,
             adjust_numerical_scale: false,
@@ -86,8 +89,9 @@ impl OptimizationSettings {
     #[args(
         verbose = "OptimizationSettings::default().verbose",
         max_memory = "OptimizationSettings::default().max_memory",
+        max_single_tensor_memory = "OptimizationSettings::default().max_single_tensor_memory",
         scheduling_num_mem_chunks = "OptimizationSettings::default().scheduling_num_mem_chunks",
-        distribute_min_size = "OptimizationSettings::default().distribute_min_size",
+        distribute_min_size = "None",
         scheduling_naive = "OptimizationSettings::default().scheduling_naive",
         scheduling_simplify = "OptimizationSettings::default().scheduling_simplify",
         adjust_numerical_scale = "OptimizationSettings::default().adjust_numerical_scale",
@@ -100,8 +104,9 @@ impl OptimizationSettings {
     fn new(
         verbose: usize,
         max_memory: usize,
+        max_single_tensor_memory: usize,
         scheduling_num_mem_chunks: usize,
-        distribute_min_size: usize,
+        distribute_min_size: Option<usize>,
         scheduling_naive: bool,
         scheduling_simplify: bool,
         adjust_numerical_scale: bool,
@@ -114,6 +119,7 @@ impl OptimizationSettings {
         Self {
             verbose,
             max_memory,
+            max_single_tensor_memory,
             scheduling_num_mem_chunks,
             distribute_min_size,
             scheduling_naive,
@@ -134,7 +140,11 @@ pub struct OptimizationCache {
     pub distributed: HashMap<HashBytes, Option<CircuitRc>>,
     pub flops: HashMap<HashBytes, BigUint>,
     pub sum_of_node_sizes: HashMap<HashBytes, BigUint>,
+    pub canonicalized: HashMap<HashBytes, Option<CircuitRc>>,
+    pub times_distributed: usize,
     pub simplification_log: Vec<&'static str>,
+    pub max_tensor_elements: usize,
+    pub max_single_tensor_numel: usize,
 }
 
 #[derive(Debug, Clone, Default)]
@@ -149,6 +159,16 @@ impl OptimizationContext {
             settings,
         }
     }
+    pub fn new_settings_circuit(settings: OptimizationSettings, circuit: CircuitRc) -> Self {
+        let mut result = Self {
+            cache: Default::default(),
+            settings,
+        };
+        let dtype = get_compatible_dtype(&circuit);
+        result.cache.max_tensor_elements = settings.max_memory / dtype.size();
+        result.cache.max_single_tensor_numel = settings.max_single_tensor_memory / dtype.size();
+        result
+    }
 }
 
 #[pyfunction]
@@ -164,7 +184,7 @@ pub fn optimize_circuit_py(circuit: CircuitRc, settings: OptimizationSettings) -
 pub fn optimize_circuit(circuit: CircuitRc, context: &mut OptimizationContext) -> CircuitRc {
     let print_timings = context.settings.verbose >= 2;
     let circuit = timed!(strip_names(circuit), 10, print_timings);
-    let circuit = timed!(deep_canonicalize(circuit), 10, print_timings);
+    let circuit = timed!(deep_canonicalize(circuit, context), 10, print_timings);
     if context.settings.verbose >= 4 {
         println!("Original Circuit");
         circuit.compiler_print();
@@ -184,7 +204,7 @@ pub fn optimize_circuit(circuit: CircuitRc, context: &mut OptimizationContext) -
     // for unknown reasons, maybe i'll investigate
     let circuit = timed!(deep_push_down_index(circuit, None), 10, print_timings);
     let circuit = timed!(compiler_simp(&circuit, context), 10, print_timings);
-    let circuit = timed!(deep_canonicalize(circuit), 10, print_timings);
+    let circuit = timed!(deep_canonicalize(circuit, context), 10, print_timings);
     if context.settings.verbose >= 3 {
         println!("Simplified Circuit");
         circuit.compiler_print();
@@ -198,12 +218,17 @@ pub fn optimize_circuit(circuit: CircuitRc, context: &mut OptimizationContext) -
         print_timings
     );
 
-    let circuit = timed!(deep_canonicalize(circuit), 10, print_timings);
+    let circuit = timed!(deep_canonicalize(circuit, context), 10, print_timings);
+    if context.settings.verbose >= 2 {
+        println!("Distributed Circuit");
+        circuit.compiler_print();
+    }
     let circuit = timed!(deep_heuristic_nest_adds(circuit), 10, print_timings);
+    let circuit = timed!(deep_canonicalize(circuit, context), 10, print_timings);
 
-    let circuit = timed!(deep_optimize_einsums(circuit), 10, print_timings);
+    let circuit = timed!(deep_optimize_einsums(circuit, context), 10, print_timings);
 
-    let circuit = timed!(deep_canonicalize(circuit), 10, print_timings);
+    let circuit = timed!(deep_canonicalize(circuit, context), 10, print_timings);
     if context.settings.verbose >= 3 {
         println!("Final Circuit");
         circuit.compiler_print();
@@ -216,10 +241,7 @@ pub fn optimize_to_schedule(
     circuit: CircuitRc,
     settings: OptimizationSettings,
 ) -> Result<Schedule, SchedulingError> {
-    let mut context = OptimizationContext {
-        settings,
-        cache: Default::default(),
-    };
+    let mut context = &mut OptimizationContext::new_settings_circuit(settings, circuit.clone());
     let verbose = settings.verbose;
     if verbose > 0 {
         println!("Optimizing verbose {}", verbose)
@@ -233,7 +255,7 @@ pub fn optimize_to_schedule(
         circuit_to_schedule_naive_toposort(optimized_circuit)
     } else {
         timed!(
-            circuit_to_schedule(optimized_circuit, settings),
+            circuit_to_schedule(optimized_circuit, context),
             10,
             verbose >= 1
         )?
