@@ -247,7 +247,6 @@ impl Einsum {
     }
 
     pub fn scalar_mul(node: &CircuitRc, scalar: f64, name: Option<String>) -> Self {
-        let name_definite = name.unwrap_or_else(|| "scalar_mul".to_owned());
         let axes: EinsumAxes = (0..node.info().rank()).map(|x| x as u8).collect();
         Einsum::try_new(
             vec![
@@ -255,7 +254,7 @@ impl Einsum {
                 (ScalarConstant::new(scalar, sv![], None).rc(), sv![]),
             ],
             axes,
-            Some(name_definite),
+            name,
         )
         .unwrap()
     }
