@@ -241,7 +241,7 @@ pub fn deep_push_down_index(circ: CircuitRc, min_size: Option<usize>) -> Circuit
 /// for each add, greedily nest into preexisting adds
 #[pyfunction]
 pub fn deep_heuristic_nest_adds(circ: CircuitRc) -> CircuitRc {
-    let circ = deep_canonicalize(circ);
+    let circ = deep_canonicalize(circ, &mut Default::default());
     let mut seen_adds: HashSet<Add> = HashSet::new();
     visit_circuit(&circ, &mut |c: &Circuit| {
         if let Some(add) = c.as_add() {
