@@ -124,7 +124,8 @@ pub fn einsum_pull_scatter(einsum: &Einsum) -> Option<CircuitRc> {
         Some(
             Scatter::try_new(
                 new_einsum,
-                uslice_map_get_index(&einsum.out_axes, &int_slices),
+                uslice_map_get_index(&einsum.out_axes, &int_slices)
+                    .canonicalize(&einsum.info().shape),
                 einsum.info().shape.clone(),
                 None,
             )
@@ -254,6 +255,10 @@ pub fn index_einsum_to_scatter(node: &Index) -> Option<CircuitRc> {
             let new_einsum = Einsum::try_new(new_args, inner.out_axes.clone(), inner.name_cloned())
                 .unwrap()
                 .rc();
+            let new_shape = containing_uslices
+                .iter()
+                .map(|x| x.unwrap().stop - x.unwrap().start)
+                .collect();
             let new_scatter = Scatter::try_new(
                 new_einsum,
                 TensorIndex(
@@ -268,11 +273,9 @@ pub fn index_einsum_to_scatter(node: &Index) -> Option<CircuitRc> {
                                 .unwrap_or(TensorAxisIndex::IDENT)
                         })
                         .collect(),
-                ),
-                containing_uslices
-                    .iter()
-                    .map(|x| x.unwrap().stop - x.unwrap().start)
-                    .collect(),
+                )
+                .canonicalize(&new_shape),
+                new_shape,
                 None,
             )
             .unwrap()
