@@ -139,7 +139,8 @@ pub fn schedule_dag_strategy_ints(
     mem_limit: usize,
     num_mem_chunks: usize,
 ) -> Result<Vec<usize>, SchedulingError> {
-    let cfg = Config::new();
+    let mut cfg = Config::new();
+    cfg.set_timeout_msec(5_000);
     let ctx = Context::new(&cfg);
     let opt = Solver::new(&ctx);
 
@@ -216,21 +217,6 @@ pub fn schedule_dag_strategy_ints(
                     )
                 })
                 .collect();
-            let end_steps: Vec<(u64, usize)> = (0..dag.node_costs.len())
-                .map(|i| {
-                    (
-                        model.eval(&end_times[i], true).unwrap().as_u64().unwrap(),
-                        i,
-                    )
-                })
-                .collect();
-            if verbose >= 3 {
-                println!(
-                    "start steps {:?} end steps {:?}",
-                    start_steps.iter().map(|x| x.1).collect::<Vec<usize>>(),
-                    end_steps.iter().map(|x| x.1).collect::<Vec<usize>>()
-                );
-            }
             start_steps.sort();
             start_steps.iter().map(|x| x.1).collect()
         })
