@@ -56,16 +56,16 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     use circuit::algebraic_rewrite::{
         add_collapse_scalar_inputs, add_deduplicate, add_elim_zeros, add_flatten_once,
         add_make_broadcasts_explicit, add_pull_removable_axes, concat_elim_identity,
-        concat_merge_uniform, concat_pull_removable_axes, concat_repeat_to_rearrange,
-        deep_optimize_einsums, distribute, distribute_all, einsum_elim_identity, einsum_elim_zero,
-        einsum_flatten_once, einsum_merge_scalars, einsum_nest_optimize, einsum_nest_path,
-        einsum_of_permute_merge, einsum_pull_removable_axes, extract_add,
-        generalfunction_pull_removable_axes, index_elim_identity, index_fuse, index_merge_scalar,
-        index_split_axes, permute_of_einsum_merge, push_down_index, rearrange_elim_identity,
-        rearrange_fuse, rearrange_merge_scalar, remove_add_few_input,
+        concat_merge_uniform, concat_pull_removable_axes, concat_repeat_to_rearrange, distribute,
+        distribute_all, einsum_elim_identity, einsum_elim_zero, einsum_flatten_once,
+        einsum_merge_scalars, einsum_nest_path, einsum_of_permute_merge,
+        einsum_pull_removable_axes, extract_add, generalfunction_pull_removable_axes,
+        index_elim_identity, index_fuse, index_merge_scalar, index_split_axes,
+        permute_of_einsum_merge, push_down_index, rearrange_elim_identity, rearrange_fuse,
+        rearrange_merge_scalar, remove_add_few_input,
     };
     use circuit::batching::{batch_inputs_axis_len, batch_largest};
-    use circuit::canonicalize::{canonicalize_node_py, deep_canonicalize};
+    use circuit::canonicalize::{canonicalize_node_py, deep_canonicalize_py};
     use circuit::circuit_manipulation::{
         deep_map_preorder_py, deep_map_py, filter_nodes_py, path_get, replace_nodes_py,
         update_nodes_py, update_path_py,
@@ -160,7 +160,7 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_function(wrap_pyfunction!(compiler_simp_py, m)?)?;
     m.add_function(wrap_pyfunction!(compiler_simp_step_py, m)?)?;
     m.add_function(wrap_pyfunction!(compiler_simp_until_same, m)?)?;
-    m.add_function(wrap_pyfunction!(deep_canonicalize, m)?)?;
+    m.add_function(wrap_pyfunction!(deep_canonicalize_py, m)?)?;
     m.add_function(wrap_pyfunction!(canonicalize_node_py, m)?)?;
 
     m.add_function(wrap_pyfunction!(strip_names, m)?)?;
@@ -170,8 +170,14 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         m
     )?)?;
     m.add_function(wrap_pyfunction!(einsum_nest_path, m)?)?;
-    m.add_function(wrap_pyfunction!(einsum_nest_optimize, m)?)?;
-    m.add_function(wrap_pyfunction!(deep_optimize_einsums, m)?)?;
+    m.add_function(wrap_pyfunction!(
+        circuit::algebraic_rewrite::einsum_nest_optimize_py,
+        m
+    )?)?;
+    m.add_function(wrap_pyfunction!(
+        circuit::algebraic_rewrite::deep_optimize_einsums_py,
+        m
+    )?)?;
 
     m.add_function(wrap_pyfunction!(index_einsum_to_scatter, m)?)?;
     m.add_function(wrap_pyfunction!(scatter_elim_identity, m)?)?;
