@@ -388,11 +388,11 @@ impl EinsumSpec {
         check_outer: Option<bool>,
         mem_cap: Option<usize>,
         hash_map_cap: Option<usize>,
-    ) -> Vec<Vec<usize>> {
+    ) -> Option<Vec<Vec<usize>>> {
         let hash_map_cap = hash_map_cap.unwrap_or(500);
-        let mut mem_cap = mem_cap.unwrap_or(usize::MAX);
+        let mem_cap = mem_cap.unwrap_or(usize::MAX);
         if self.input_ints.len() == 1 {
-            return vec![vec![0]];
+            return Some(vec![vec![0]]);
         }
 
         let tensor_ints: Vec<Vec<usize>> = self.input_ints.clone();
@@ -437,10 +437,6 @@ impl EinsumSpec {
                 .into_iter()
                 .collect();
             while dp.last().unwrap().is_empty() {
-                if mem_cap == usize::MAX && cost_cap == usize::MAX {
-                    // give up
-                    return vec![vec![0, 1]; tensor_ints.len() - 1];
-                }
                 for n in 2..dp[1].len() + 1 {
                     // need this bc iter earier idxs while updating later
                     let (dp_prev, dp_here_vec) = dp.split_at_mut(n);
@@ -524,7 +520,7 @@ impl EinsumSpec {
                 }
                 let new_cost_checked = cost_cap.checked_mul(cost_cap_increment);
                 match new_cost_checked {
-                    None => mem_cap = mem_cap.checked_mul(2).unwrap_or(usize::MAX),
+                    None => return None,
                     Some(x) => cost_cap = x,
                 }
             }
@@ -548,7 +544,7 @@ impl EinsumSpec {
         );
         let result = contraction_ssa_ids_to_recycled(&ssa_result, tensor_ints.len());
         assert!(&result.iter().all(|x| x.len() == 2), "{:?}", &result);
-        result
+        Some(result)
     }
 
     pub fn optimize(
@@ -557,7 +553,7 @@ impl EinsumSpec {
         mem_cap: Option<usize>,
         hash_map_cap: Option<usize>,
         dont_contract_subsets_into: Option<Vec<usize>>,
-    ) -> Vec<Vec<usize>> {
+    ) -> Option<Vec<Vec<usize>>> {
         let (mut result, new_input_ints) =
             self.optimize_pre_contract_subsets(dont_contract_subsets_into);
         // this is ugly, todo fix
@@ -574,9 +570,9 @@ impl EinsumSpec {
                 output_ints: self.output_ints.clone(),
                 int_sizes: self.int_sizes.clone(),
             };
-            result.extend(new_spec.optimize_dp(check_outer, mem_cap, hash_map_cap));
+            result.extend(new_spec.optimize_dp(check_outer, mem_cap, hash_map_cap)?);
         }
-        result
+        Some(result)
     }
 
     fn __repr__(&self) -> String {
@@ -620,7 +616,7 @@ pub fn optimize_einsum_spec_cached(
     mem_cap: Option<usize>,
     hash_map_cap: Option<usize>,
     dont_contract_subsets_into: Option<Vec<usize>>,
-) -> Vec<Vec<usize>> {
+) -> Option<Vec<Vec<usize>>> {
     spec.optimize(
         check_outer,
         mem_cap,
