@@ -581,6 +581,12 @@ impl TorchDeviceDtypeOp {
     fn default_py() -> Self {
         Self::default()
     }
+
+    #[new]
+    #[args(device = "None", dtype = "None")]
+    fn new(device: Option<String>, dtype: Option<String>) -> Self {
+        Self { device, dtype }
+    }
 }
 
 #[pyclass]
