@@ -1,7 +1,7 @@
 use rand::Rng;
 
 use rust_circuit::circuit::diag_rewrite::{diags_intersection, diags_union};
-use rust_circuit::circuit::{CircuitNode, GeneralFunction};
+use rust_circuit::circuit::{get_compatible_dtype, CircuitNode, GeneralFunction};
 use rust_circuit::opt_einsum::{
     get_disconnected_sugraphs, get_int_to_tensor_appearance, EinsumSpec,
 };
@@ -151,6 +151,7 @@ fn test_generalfuction() {
         sv![0, 1],
         None,
     );
+    println!("{:?}", get_compatible_dtype(&circuit));
     optimize_and_evaluate(circuit, Default::default()).unwrap();
 }
 
